/**
 * Debugging class.
 */
;(function(){

	window.tantrum = function(context, task, message)
	{
		this.context 	= context;
		this.task 		= task;
		this.message 	= message;
		bug.error('EXCEPTION on ' + context, task, message);
	};

	window.bug = {


		/**
		 * Debug level.
		 *  0 : No logging
		 *  1 : Errors only
		 *  2 : Errors and warnings
		 *  3 : Errors, warnings and information
		 *  4 : Errors, warnings, information and dumps
		 *  5 : Flood mode, hope you can swim
		 * @type {Number}
		 */
		_debugLevel : 3,


		/**
		 * Set debug verbosity level.
		 * @param {Number} level Debug verbosity level
		 */
		setLevel : function(level)
		{
			bug._debugLevel = level;
		},

		/**
		 * Start a logging group.
		 * @param  {String} groupName Name of the group
		 */
		group : function(groupName)
		{
			console.groupCollapsed(groupName);
		},

		/**
		 * End the current logging group.
		 */
		ungroup : function()
		{
			console.groupEnd();
		},

		/**
		 * Log an error.
		 * @param  {String}  context Module, class or object context
		 * @param  {String}  task    The task
		 * @param  {Unknown} data    The data to log
		 */
		error : function(context, task, data)
		{
			if (0 < bug._debugLevel)
			{
				bug._handle('', 'error', context, task, data);
			}
		},
		
		/**
		 * Log a warning.
		 * @param  {String}  context Module, class or object context
		 * @param  {String}  task    The task
		 * @param  {Unknown} data    The data to log
		 */
		warn : function(context, task, data)
		{
			if (1 < bug._debugLevel)
			{
				bug._handle('', 'warn', context, task, data);
			}
		},
		debug : function(context, task, data)
		{
			if (1 < bug._debugLevel)
			{
				bug._handle('-oO- ', 'warn', context, task, data);
			}
		},
		
		/**
		 * Log some information.
		 * @param  {String}  context Module, class or object context
		 * @param  {String}  task    The task
		 * @param  {Unknown} data    The data to log
		 */
		info : function(context, task, data)
		{
			if (2 < bug._debugLevel)
			{
				bug._handle('(i) ', 'log', context, task, data);
			}
		},
		
		/**
		 * Dump some information, probably lots of it.
		 * @param  {String}  context Module, class or object context
		 * @param  {String}  task    The task
		 * @param  {Unknown} data    The data to log
		 */
		dump : function(context, task, data)
		{
			if (3 < bug._debugLevel)
			{
				bug._handle('<dump> ', 'log', context, task, data);
			}
		},
		
		/**
		 * Drown the developer in tons of information.
		 * @param  {String}  context Module, class or object context
		 * @param  {String}  task    The task
		 * @param  {Unknown} data    The data to log
		 */
		drown : function(context, task, data)
		{
			if (4 < bug._debugLevel)
			{
				bug._handle('..... ', 'log', context, task, data);
			}
		},

		/**
		 * Internal utility to do the actual logging.
		 * @param  {String}  level   Logging level
		 * @param  {String}  method  Console method to use
		 * @param  {String}  context Module, class or object context
		 * @param  {String}  task    The task
		 * @param  {Unknown} data    The data to log
		 */
		_handle : function(level, method, context, task, data, wrap)
		{
			if (undefined == data && undefined == task)
			{
				console[method](level, context);
			}
			else if (undefined == data)
			{
				console[method](level + context, task);
			}
			else
			{
				console[method](level + context + ':' + task, data);
			}
		}


	};

})();